@@ -200,6 +200,24 @@ class TestAclOps(OvnNorthboundTest):
         self._acl_add('port_group', 'from-lport', 0,
                       'output == "fake_port" && ip', 'drop')
 
+    def test_pg_acl_add_logging(self):
+        r1 = self._acl_add('port_group', 'from-lport', 0,
+                           'output == "fake_port" && ip', 'drop',
+                           log=True, severity="warning", name="test1",
+                           meter="meter1", extkey1="extval1")
+        self.assertTrue(r1.log)
+        self.assertEqual(["meter1"], r1.meter)
+        self.assertEqual(["warning"], r1.severity)
+        self.assertEqual(["test1"], r1.name)
+        self.assertEqual({"extkey1": "extval1"}, r1.external_ids)
+
+    def test_pg_acl_add_logging_weird_severity(self):
+        r1 = self._acl_add('port_group', 'from-lport', 0,
+                           'output == "fake_port" && ip', 'drop',
+                           severity="weird")
+        self.assertNotEqual(["weird"], r1.severity)
+        self.assertEqual([], r1.severity)
+
     def test_pg_acl_del_all(self):
         r1 = self._acl_add('port_group', 'from-lport', 0,
                            'output == "fake_port"', 'drop')
@@ -676,6 +694,9 @@ class TestDhcpOptionsOps(OvnNorthboundTest):
 
 
 class TestLogicalRouterOps(OvnNorthboundTest):
+    lr_policy_match1 = 'ip4.src==10.1.1.0/24 && ip4.dst==20.1.1.0/24'
+    lr_policy_match2 = 'ip4.src==30.1.1.0/24 && ip4.dst==40.1.1.0/24'
+
     def _lr_add(self, *args, **kwargs):
         lr = self.useFixture(
             fixtures.LogicalRouterFixture(self.api, *args, **kwargs)).obj
@@ -718,11 +739,13 @@ class TestLogicalRouterOps(OvnNorthboundTest):
                           may_exist=True)
         prefix = prefix or '192.0.2.0/25'
         nexthop = nexthop or '192.0.2.254'
+        port = port or "port_name"
         sr = self.api.lr_route_add(lr.uuid, prefix, nexthop, port,
                                    **kwargs).execute(check_error=True)
         self.assertIn(sr, lr.static_routes)
         self.assertEqual(prefix, sr.ip_prefix)
         self.assertEqual(nexthop, sr.nexthop)
+        self.assertIn(port, sr.output_port)
         sr.router = lr
         return sr
 
@@ -947,6 +970,153 @@ class TestLogicalRouterOps(OvnNorthboundTest):
         nats = self.api.lr_nat_list(lr.uuid).execute(check_error=True)
         self.assertEqual(lr.nat, nats)
 
+    def _lr_policy_add(self, priority, match, action, *args, **kwargs):
+        lr = kwargs.pop('router', self._lr_add(utils.get_rand_device_name()))
+        policy = self.api.lr_policy_add(
+            lr.uuid, priority, match, action, *args, **kwargs).execute(
+            check_error=True)
+        self.assertIn(policy, lr.policies)
+        self.assertEqual(action, policy.action)
+        self.assertEqual(match, policy.match)
+        self.assertEqual(priority, policy.priority)
+        policy.router = lr
+        return policy
+
+    def test_lr_policy_add_allow(self):
+        priority = 10
+        action = const.POLICY_ACTION_ALLOW
+        policy = self._lr_policy_add(priority, self.lr_policy_match1, action)
+        self.assertEqual(policy.nexthop, [])
+        self.assertEqual(policy.nexthops, [])
+
+    def test_lr_policy_add_drop(self):
+        priority = 10
+        action = const.POLICY_ACTION_DROP
+        policy = self._lr_policy_add(priority, self.lr_policy_match1, action)
+        self.assertEqual(policy.nexthop, [])
+        self.assertEqual(policy.nexthops, [])
+
+    def test_lr_policy_add_reroute_nexthop(self):
+        priority = 10
+        action = const.POLICY_ACTION_REROUTE
+        nexthop = "10.3.0.2"
+        policy = self._lr_policy_add(priority, self.lr_policy_match1, action,
+                                     nexthop=nexthop)
+        self.assertEqual(policy.nexthop, [nexthop])
+
+    def test_lr_policy_add_reroute_nexthops(self):
+        priority = 10
+        action = const.POLICY_ACTION_REROUTE
+        nexthops = ["10.3.0.2", "10.3.0.3"]
+        policy = self._lr_policy_add(priority, self.lr_policy_match1, action,
+                                     nexthops=nexthops)
+        self.assertEqual(policy.nexthops, nexthops)
+
+    def test_lr_policy_add_invalid_action(self):
+        priority = 10
+        action = "allow-related"
+        self.assertRaises(TypeError, self._lr_policy_add,
+                          priority, self.lr_policy_match1, action)
+
+    def test_lr_policy_add_invalid_priority(self):
+        priority = -1
+        action = "allow"
+        self.assertRaises(ValueError, self._lr_policy_add,
+                          priority, self.lr_policy_match1, action)
+
+    def test_lr_policy_add_reroute_no_nexthop(self):
+        priority = 10
+        action = const.POLICY_ACTION_REROUTE
+        self.assertRaises(ValueError, self._lr_policy_add,
+                          priority, self.lr_policy_match1, action)
+
+    def test_lr_policy_add_nexthop_not_reroute(self):
+        priority = 10
+        action = const.POLICY_ACTION_ALLOW
+        nexthop = "10.3.0.2"
+        self.assertRaises(ValueError, self._lr_policy_add, priority,
+                          self.lr_policy_match1, action, nexthop=nexthop)
+
+    def test_lr_policy_add_nexthops_not_reroute(self):
+        priority = 10
+        action = const.POLICY_ACTION_ALLOW
+        nexthops = ["10.3.0.2", "10.3.0.3"]
+        self.assertRaises(ValueError, self._lr_policy_add, priority,
+                          self.lr_policy_match1, action, nexthops=nexthops)
+
+    def test_lr_policy_may_exist(self):
+        priority = 10
+        action = const.POLICY_ACTION_ALLOW
+        policy_init = self._lr_policy_add(priority, self.lr_policy_match1,
+                                          action)
+        self.assertEqual(policy_init.nexthop, [])
+
+        # Update the policy
+        action = const.POLICY_ACTION_REROUTE
+        nexthop = "10.3.0.2"
+        policy_new = self._lr_policy_add(priority, self.lr_policy_match1,
+                                         action, nexthop=nexthop,
+                                         may_exist=True,
+                                         router=policy_init.router)
+        self.assertEqual(policy_init, policy_new)
+        self.assertEqual(policy_new.nexthop, [nexthop])
+
+    def _three_policies(self):
+        lr = self._lr_add(utils.get_rand_device_name())
+        pr1 = 10
+        pr2 = 9
+        self._lr_policy_add(pr1, self.lr_policy_match1,
+                            const.POLICY_ACTION_ALLOW, router=lr)
+        self._lr_policy_add(pr2, self.lr_policy_match2,
+                            const.POLICY_ACTION_DROP, router=lr)
+        self._lr_policy_add(pr1, self.lr_policy_match2,
+                            const.POLICY_ACTION_REROUTE,
+                            nexthops=['31.1.1.2', '31.1.1.3'], router=lr)
+        return lr
+
+    def _lr_policy_del(self, *args, **kwargs):
+        lr = self._three_policies()
+        self.api.lr_policy_del(lr.name,
+                               *args,
+                               **kwargs).execute(check_error=True)
+        return lr
+
+    def test_lr_policy_del_all(self):
+        lr = self._lr_policy_del()
+        self.assertEqual(lr.policies, [])
+
+    def test_lr_policy_del_priority(self):
+        priority = 10
+        lr = self._lr_policy_del(priority=priority)
+        self.assertEqual(len(lr.policies), 1)
+        self.assertEqual(lr.policies[0].action, const.POLICY_ACTION_DROP)
+
+    def test_lr_policy_del_match_without_priority(self):
+        self.assertRaises(TypeError, self._lr_policy_del,
+                          match='ip4.dst==40.1.1.0/24')
+
+    def test_lr_policy_del_specific(self):
+        priority = 10
+        lr = self._lr_policy_del(match=self.lr_policy_match2,
+                                 priority=priority)
+        for policy in lr.policies:
+            self.assertNotEqual(policy.action, const.POLICY_ACTION_REROUTE)
+            self.assertEqual(policy.nexthops, [])
+
+    def test_lr_policy_del_not_found(self):
+        self.assertRaises(RuntimeError, self._lr_policy_del,
+                          priority=100, match='ip4.dst==40.1.1.0/24')
+
+    def test_lr_policy_del_if_exists(self):
+        lr = self._lr_policy_del(priority=100, match='ip4.dst==40.1.1.0/24',
+                                 if_exists=True)
+        self.assertEqual(len(lr.policies), 3)
+
+    def test_lr_policy_list(self):
+        lr = self._three_policies()
+        policies = self.api.lr_policy_list(lr.uuid).execute(check_error=True)
+        self.assertEqual(lr.policies, policies)
+
 
 class TestLogicalRouterPortOps(OvnNorthboundTest):
     def setUp(self):
@@ -1048,6 +1218,15 @@ class TestLogicalRouterPortOps(OvnNorthboundTest):
         self.assertIn(c1, lrp.gateway_chassis)
         self.assertIn(c2, lrp.gateway_chassis)
 
+    def test_gwc_add(self):
+        # NOTE: no API method to create gateway chassis directly
+        name, c1_name = [utils.get_rand_device_name() for _ in range(2)]
+        lrp = self._lrp_add(name, gateway_chassis=[c1_name])
+        c1 = self.api.lookup('Gateway_Chassis', "%s_%s" % (lrp.name, c1_name))
+        self.assertEqual(c1.name, "%s_%s" % (lrp.name, c1_name))
+        self.assertEqual(c1.chassis_name, c1_name)
+        self.assertEqual(c1.priority, 1)
+
     def test_lrp_del_uuid(self):
         lrp = self._lrp_add(None)
         self.api.lrp_del(lrp.uuid).execute(check_error=True)
@@ -1691,3 +1870,89 @@ class TestReferencedObjects(OvnNorthboundTest):
             txn.add(self.api.db_add('Logical_Switch_Port', prt,
                                     "addresses", self.lsp_test_addresses[0]))
         self._check_values()
+
+
+class TestMeterOps(OvnNorthboundTest):
+
+    def setUp(self):
+        super(TestMeterOps, self).setUp()
+        self.table = self.api.tables['Meter']
+
+    def _meter_add(self, name=None, unit=None, rate=1, fair=False,
+                   burst_size=0, action=None, may_exist=False, **columns):
+        if name is None:
+            name = utils.get_rand_name()
+        if unit is None:
+            unit = 'pktps'
+        # NOTE(flaviof): action drop is the only one option in the schema when
+        #                this test was implemented. That is expected to be
+        #                properly handled by api.
+        exp_action = 'drop' if action is None else action
+        meter = self.useFixture(fixtures.MeterFixture(
+            self.api, name, unit, rate, fair, burst_size, action, may_exist,
+            **columns)).obj
+        meter_band = meter.bands[0]
+        self.assertEqual(name, meter.name)
+        self.assertEqual(unit, meter.unit)
+        self.assertEqual([fair], meter.fair)
+        self.assertEqual(exp_action, meter_band.action)
+        self.assertEqual(rate, meter_band.rate)
+        self.assertEqual(burst_size, meter_band.burst_size)
+        return meter
+
+    def _meter_del(self, row, col):
+        name404 = utils.get_rand_name()
+        self.api.meter_del(name404, if_exists=True).execute(check_error=True)
+        self.assertIn(row.uuid, self.table.rows)
+        self.api.meter_del(row.uuid).execute(check_error=True)
+        self.assertNotIn(row.uuid, self.table.rows)
+        self.api.meter_del(col, if_exists=True).execute(check_error=True)
+        cmd = self.api.meter_del(col)
+        self.assertRaises(RuntimeError, cmd.execute, check_error=True)
+
+    def _meter_get(self, col):
+        meter = self._meter_add()
+        val = getattr(meter, col)
+        found = self.api.meter_get(val).execute(check_error=True)
+        self.assertEqual(meter, found)
+
+    def test_meter_add_non_defaults(self):
+        self._meter_add(utils.get_rand_name(), "kbps", 321, True, 500, None)
+
+    def test_meter_add_ext_ids(self):
+        ext_ids = {
+            utils.get_rand_name(prefix="random_"): utils.get_random_string(10)
+            for _ in range(3)}
+        meter = self._meter_add(external_ids=ext_ids)
+        self.assertEqual(ext_ids, meter.external_ids)
+
+    def test_meter_add_may_exist(self):
+        cmd = self.api.meter_add("same_name", "kbps", may_exist=True)
+        m1 = cmd.execute(check_error=True)
+        m2 = cmd.execute(check_error=True)
+        self.assertEqual(m1, m2)
+
+    def test_meter_add_duplicate(self):
+        cmd = self.api.meter_add("same_name", "kbps", may_exist=False)
+        cmd.execute(check_error=True)
+        self.assertRaises(RuntimeError, cmd.execute, check_error=True)
+
+    def test_meter_del_by_uuid(self):
+        meter = self._meter_add()
+        self._meter_del(meter, meter.uuid)
+
+    def test_meter_del_by_name(self):
+        name = utils.get_rand_name()
+        meter = self._meter_add(name)
+        self._meter_del(meter, name)
+
+    def test_meter_list(self):
+        meters = {self._meter_add() for _ in range(3)}
+        meter_set = set(self.api.meter_list().execute(check_error=True))
+        self.assertTrue(meters.issubset(meter_set))
+
+    def test_meter_get_uuid(self):
+        self._meter_get('uuid')
+
+    def test_meter_get_name(self):
+        self._meter_get('name')
