@@ -28,8 +28,12 @@ DUMMY_OVERRIDE_NONE = ''
 
 class OvsVenvFixture(fixtures.Fixture):
     PATH_VAR_TEMPLATE = "{0}/ovsdb:{0}/vswitchd:{0}/utilities"
+    OVS_PATHS = (
+        os.path.join(os.path.sep, 'usr', 'local', 'share', 'openvswitch'),
+        os.path.join(os.path.sep, 'usr', 'share', 'openvswitch'))
 
-    def __init__(self, venv, ovsdir, dummy=DUMMY_OVERRIDE_ALL, remove=False):
+    def __init__(self, venv, ovsdir=None, dummy=DUMMY_OVERRIDE_ALL,
+                 remove=False):
         """Initialize fixture
 
         :param venv: Path to venv directory.
@@ -42,21 +46,34 @@ class OvsVenvFixture(fixtures.Fixture):
         self.venv = venv
         self.env = {'OVS_RUNDIR': self.venv, 'OVS_LOGDIR': self.venv,
                     'OVS_DBDIR': self.venv, 'OVS_SYSCONFDIR': self.venv}
-        if ovsdir:
-            self.ovsdir = ovsdir
-            self.env['PATH'] = (self.PATH_VAR_TEMPLATE.format(self.ovsdir) +
+        if ovsdir and os.path.isdir(ovsdir):
+            # From source directory
+            self.env['PATH'] = (self.PATH_VAR_TEMPLATE.format(ovsdir) +
                                 ":%s" % os.getenv('PATH'))
         else:
+            # Use installed OVS
             self.env['PATH'] = os.getenv('PATH')
-            self.ovsdir = os.path.join('/usr', 'local', 'share', 'openvswitch')
-            if not os.path.isdir(self.ovsdir):
-                self.ovsdir = os.path.join('/usr', 'share', 'openvswitch')
-        if not os.path.isdir(self.ovsdir):
-            raise Exception("%s is not a directory" % self.ovsdir)
+
+        self.ovsdir = self._share_path(self.OVS_PATHS, ovsdir)
         self._dummy = dummy
         self.remove = remove
         self.ovsdb_server_dbs = []
 
+    @staticmethod
+    def _share_path(paths, override=None, files=tuple()):
+        if not override:
+            try:
+                return next(
+                    p for p in paths if os.path.isdir(p) and
+                    all(os.path.isfile(os.path.join(p, f)) for f in files))
+            except StopIteration:
+                pass
+        elif os.path.isdir(override):
+            return override
+
+        raise Exception("Invalid directories: %s" %
+                        ", ".join(paths + (str(override),)))
+
     @property
     def ovs_schema(self):
         path = os.path.join(self.ovsdir, 'vswitchd', 'vswitch.ovsschema')
@@ -73,7 +90,7 @@ class OvsVenvFixture(fixtures.Fixture):
         return 'unix:' + os.path.join(self.venv, 'db.sock')
 
     def _setUp(self):
-        super(OvsVenvFixture, self)._setUp()
+        super()._setUp()
         self.addCleanup(self.deactivate)
         if not os.path.isdir(self.venv):
             os.mkdir(self.venv)
@@ -113,7 +130,8 @@ class OvsVenvFixture(fixtures.Fixture):
     def call(self, cmd, *args, **kwargs):
         cwd = kwargs.pop('cwd', self.venv)
         return subprocess.check_call(
-            cmd, *args, env=self.env, cwd=cwd, **kwargs)
+            cmd, *args, env=self.env, stderr=subprocess.STDOUT,
+            cwd=cwd, **kwargs)
 
     def get_pids(self):
         files = glob.glob(os.path.join(self.venv, "*.pid"))
@@ -129,29 +147,38 @@ class OvsVenvFixture(fixtures.Fixture):
 
 
 class OvsOvnVenvFixture(OvsVenvFixture):
+    OVN_PATHS = (
+        os.path.join(os.path.sep, 'usr', 'local', 'share', 'ovn'),
+        os.path.join(os.path.sep, 'usr', 'share', 'ovn')) + (
+            OvsVenvFixture.OVS_PATHS)
 
-    def __init__(self, *args, **kwargs):
-        self.add_chassis = kwargs.pop('add_chassis', False)
-        self.ovndir = kwargs.pop('ovndir') or '/usr/local/share/ovn'
-        self.PATH_VAR_TEMPLATE += (
-            ":{0}/controller:{0}/northd:{0}/utilities".format(
-                self.ovndir))
-        super(OvsOvnVenvFixture, self).__init__(*args, **kwargs)
+    NBSCHEMA = 'ovn-nb.ovsschema'
+    SBSCHEMA = 'ovn-sb.ovsschema'
+    IC_NBSCHEMA = 'ovn-ic-nb.ovsschema'
+
+    def __init__(self, venv, ovndir=None, add_chassis=False, **kwargs):
+        self.add_chassis = add_chassis
+        if ovndir and os.path.isdir(ovndir):
+            # Use OVN source dir
+            self.PATH_VAR_TEMPLATE += (
+                ":{0}/controller:{0}/northd:{0}/utilities".format(ovndir))
+        super().__init__(venv, **kwargs)
+        self.ovndir = self._share_path(self.OVN_PATHS, ovndir,
+                                       [self.SBSCHEMA, self.NBSCHEMA,
+                                        self.IC_NBSCHEMA])
         self.env.update({'OVN_RUNDIR': self.venv})
 
     @property
     def ovnsb_schema(self):
-        path = os.path.join(self.ovsdir, 'ovn', 'ovn-sb.ovsschema')
-        if os.path.isfile(path):
-            return path
-        return os.path.join(self.ovndir, 'ovn-sb.ovsschema')
+        return os.path.join(self.ovndir, self.SBSCHEMA)
 
     @property
     def ovnnb_schema(self):
-        path = os.path.join(self.ovsdir, 'ovn', 'ovn-nb.ovsschema')
-        if os.path.isfile(path):
-            return path
-        return os.path.join(self.ovndir, 'ovn-nb.ovsschema')
+        return os.path.join(self.ovndir, self.NBSCHEMA)
+
+    @property
+    def ovn_icnb_schema(self):
+        return os.path.join(self.ovndir, self.IC_NBSCHEMA)
 
     @property
     def ovnnb_connection(self):
@@ -161,38 +188,45 @@ class OvsOvnVenvFixture(OvsVenvFixture):
     def ovnsb_connection(self):
         return 'unix:' + os.path.join(self.venv, 'ovnsb_db.sock')
 
+    @property
+    def ovn_icnb_connection(self):
+        return 'unix:' + os.path.join(self.venv, 'ovn_ic_nb_db.sock')
+
     def setup_dbs(self):
-        super(OvsOvnVenvFixture, self).setup_dbs()
+        super().setup_dbs()
         self.create_db('ovnsb.db', self.ovnsb_schema)
         self.create_db('ovnnb.db', self.ovnnb_schema)
+        self.create_db('ovn_ic_nb.db', self.ovn_icnb_schema)
 
     def start_ovsdb_processes(self):
-        super(OvsOvnVenvFixture, self).start_ovsdb_processes()
-        self.call(['ovsdb-server', '--detach', '--no-chdir', '-vconsole:off',
-                   '--pidfile=%s' % os.path.join(self.venv, 'ovnnb_db.pid'),
-                   '--log-file=%s' % os.path.join(self.venv, 'ovnnb_db.log'),
-                   '--remote=db:OVN_Northbound,NB_Global,connections',
-                   '--private-key=db:OVN_Northbound,SSL,private_key',
-                   '--certificate=db:OVN_Northbound,SSL,certificate',
-                   '--ca-cert=db:OVN_Northbound,SSL,ca_cert',
-                   '--ssl-protocols=db:OVN_Northbound,SSL,ssl_protocols',
-                   '--ssl-ciphers=db:OVN_Northbound,SSL,ssl_ciphers',
-                   '--remote=p' + self.ovnnb_connection, 'ovnnb.db'])
-        self.call(['ovsdb-server', '--detach', '--no-chdir', '-vconsole:off',
-                   '--pidfile=%s' % os.path.join(self.venv, 'ovnsb_db.pid'),
-                   '--log-file=%s' % os.path.join(self.venv, 'ovnsb_db.log'),
-                   '--remote=db:OVN_Southbound,SB_Global,connections',
-                   '--private-key=db:OVN_Southbound,SSL,private_key',
-                   '--certificate=db:OVN_Southbound,SSL,certificate',
-                   '--ca-cert=db:OVN_Southbound,SSL,ca_cert',
-                   '--ssl-protocols=db:OVN_Southbound,SSL,ssl_protocols',
-                   '--ssl-ciphers=db:OVN_Southbound,SSL,ssl_ciphers',
-                   '--remote=p' + self.ovnsb_connection, 'ovnsb.db'])
+        super().start_ovsdb_processes()
+        for connection, schema, db_name, table in [
+            (self.ovnnb_connection,
+             "OVN_Northbound", "ovnnb", "NB_Global"),
+            (self.ovnsb_connection,
+             "OVN_Southbound", "ovnsb", "SB_Global"),
+            (self.ovn_icnb_connection,
+             "OVN_IC_Northbound", "ovn_ic_nb", "IC_NB_Global"),
+        ]:
+            self.call(['ovsdb-server',
+                       '--detach', '--no-chdir', '-vconsole:off',
+                       '--pidfile=%s' % os.path.join(self.venv,
+                                                     '%s_db.pid' % (db_name)),
+                       '--log-file=%s' % os.path.join(self.venv,
+                                                      '%s_db.log' % (db_name)),
+                       '--remote=db:%s,%s,connections' % (schema, table),
+                       '--private-key=db:%s,SSL,private_key' % (schema),
+                       '--certificate=db:%s,SSL,certificate' % (schema),
+                       '--ca-cert=db:%s,SSL,ca_cert' % (schema),
+                       '--ssl-protocols=db:%s,SSL,ssl_protocols' % (schema),
+                       '--ssl-ciphers=db:%s,SSL,ssl_ciphers' % (schema),
+                       '--remote=p' + connection, '%s.db' % (db_name)])
 
     def init_processes(self):
-        super(OvsOvnVenvFixture, self).init_processes()
+        super().init_processes()
         self.call(['ovn-nbctl', 'init'])
         self.call(['ovn-sbctl', 'init'])
+        self.call(['ovn-ic-nbctl', 'init'])
         if self.add_chassis:
             self.call([
                 'ovs-vsctl', 'set', 'open', '.',
@@ -201,11 +235,8 @@ class OvsOvnVenvFixture(OvsVenvFixture):
                 'external_ids:ovn-encap-type=geneve',
                 'external_ids:ovn-encap-ip=127.0.0.1'])
         # TODO(twilson) SSL stuff
-        if False:
-            pass
-        else:
-            self.call(['ovs-vsctl', 'set', 'open', '.',
-                       'external_ids:ovn-remote=' + self.ovnsb_connection])
+        self.call(['ovs-vsctl', 'set', 'open', '.',
+                   'external_ids:ovn-remote=' + self.ovnsb_connection])
         self.call(['ovn-northd', '--detach', '--no-chdir', '--pidfile',
                    '-vconsole:off', '--log-file',
                    '--ovnsb-db=' + self.ovnsb_connection,
